/*
 *            File: ClientColori.java
 *          Autore: Roberto FULIGNI
 * Ultima modifica: 17/04/2024
 *
 *     Descrizione: Software client per accedere a servizi REST protetti
 *                  mediante autenticazione basata su API key e token JWT.
 */

package edu.fauser.netlab;

import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.Form;
import javax.ws.rs.core.GenericType;
import javax.ws.rs.core.Response;
import java.util.List;

public class ClientColori {
    final static String URL_BASE = "http://localhost:8080/server-colori-jwt/api";
    private static String TOKEN_JWT = "";
    private static WebTarget targetColori;

    public static void main(String[] args) {

        Client client = ClientBuilder.newClient();
        WebTarget targetBase = client.target(URL_BASE);

        // Definizione dell'URI della risorsa obiettivo ("target")
        // In questo esempio la variabile "target" è condivisa
        // tra tutte le funzioni di elaborazione

        targetColori = targetBase.path("colori");

        // Tentativo di eseguire l'accesso a una risorsa protetta senza
        // autenticazione
        stampaElencoColori();

        // Esegue la routine di autenticazione e memorizza il token JWT
        // restituito nella variabile globale TOKEN_JWT.
        // Il token sarà utilizzato in tutte le richieste successive.

        TOKEN_JWT = autentica(targetBase);

        // Stampa del contenuto iniziale della mappa memorizzata nel server
        stampaElencoColori();

        // Aggiunge un nuovo colore alla mappa
        creaNuovoColore("Blu", "0,0,255", "#0000ff");

        // Aggiornamento di un dato colore ...
        modificaColore(3, "0,0,254", "#0000fe");

        // ... E relativa stampa su schermo
        stampaSingoloColore(3);
    }

    private static String autentica(WebTarget base) {
        // Inserisce la API KEY nel campo di una form
        Form form = new Form();
        form.param("apikey", "def456");

        // Le richieste contenenti i campi di una form includono automaticamente
        // l'intestazione "application/x-www-form-urlencoded", come richiesto
        // dal server per procedere all'autenticazione.

        Response risposta = base.path("auth").request().post(Entity.form(form));

        // Legge e restituisce la stringa inclusa nel body della risposta.
        // La stringa rappresenta il token JWT inviato dal server.
        String token = risposta.readEntity(String.class);
        return token;
    }
    private static void stampaElencoColori() {
        // si esegue una richiesta GET sulla risorsa target definita nel main (/api/colori)
        // e si memorizza la risposta
        Response r = targetColori.request()
                .header("Authorization", "Bearer " + TOKEN_JWT)
                .get();

        // Informazioni sullo stato della risposta HTTP
        System.out.format("stampaElencoColori - Status: %d (%s)%n", r.getStatus(), r.getStatusInfo().getFamily().name());

        if (r.getStatus() == 200) {
            // Legge l'informazione contenuta nel body della risposta HTTP (entità in formato JSON)
            // e la trasforma in una lista di oggetti "ColoreBreve"

            List<ColoreBreve> colori = r.readEntity(new GenericType<List<ColoreBreve>>() {});
            for (var c : colori) {
                System.out.format("Colore %s (id: %d)%n", c.getNome(), c.getId());
            }
        }
    }
    private static void creaNuovoColore(String nome, String rgb, String hex) {
        // Crea un nuovo oggetto "Colore" a partire dai dati del nuovo colore
        // L'attributo id è tralasciato in quanto determinato automaticamente dal server
        Colore c = new Colore(null,nome, rgb, hex);

        // Esegue una  richiesta POST sulla risorsa target (/api/colori)
        // Il body della richiesta include il colore da creare in formato JSON
        Response r = targetColori.request()
                .header("Authorization", "Bearer " + TOKEN_JWT)
                .post(Entity.json(c));

        System.out.format("creaNuovoColore - Status: %d (%s)%n", r.getStatus(), r.getStatusInfo().getFamily().name());
        if (r.getStatus() == 201) {
            System.out.format("Location: %s%n", r.getLocation());
        }
    }

    private static void modificaColore(Integer idColore, String rgb, String hex) {
        // Aggiornamento dei codici di colore con il metodo PUT.
        // Prima fase: si acquisisce la descrizione iniziale del colore

        Response r = targetColori.path(idColore.toString()).request()
                .header("Authorization", "Bearer " + TOKEN_JWT)
                .get();

        if (r.getStatus() != 200) {
            System.out.format("modificaColore - Status: %d (%s)%n", r.getStatus(), r.getStatusInfo().getFamily().name());
            return;
        }
        // Seconda fase: trasformazione dell'entità JSON in un oggetto Java
        // e modifica dei codici RGB e HEX
        Colore c = r.readEntity(Colore.class);
        c.setRgb(rgb);
        c.setHex(hex);

        // Terza fase: invio di una richiesta PUT per aggiornare l'intero colore
        r = targetColori.path(idColore.toString()).request()
                .header("Authorization", "Bearer " + TOKEN_JWT)
                .put(Entity.json(c));
        System.out.format("modificaColore - Status: %d (%s)%n", r.getStatus(), r.getStatusInfo().getFamily().name());
    }
    private static void stampaSingoloColore(Integer idColore) {
        // Acquisizione della descrizione del colore dalla risorsa target /api/colori/{idColore}
        Response r = targetColori.path(idColore.toString()).request()
                .header("Authorization", "Bearer " + TOKEN_JWT)
                .get();

        System.out.format("stampaSingoloColore - Status: %d (%s)%n", r.getStatus(), r.getStatusInfo().getFamily().name());
        if (r.getStatus() == 200) {
            Colore c = r.readEntity(Colore.class);
            System.out.printf("Id: %d  NomeColore: %s   Rgb: %s  Hex: %s%n", c.getId(), c.getNome(), c.getRgb(), c.getHex());
        }
    }
}